#! /bin/bash
#
# Edit configuration
#

# Default parameter
: ${edit_config_method:=menuconfig}

# Global configuration variables
: ${possible_terminals:=gnome-terminal xfce rxvt konsole xterm}
: ${default_terminal:=auto}

# Default variables
: ${package_name:=$PACKAGE_NAME}
: ${package_version:=$PACKAGE_VERSION}
: ${package_revision:=$PACKAGE_REVISION}
: ${package_revision:=$PACKAGE_REV}
: ${package_statedir:=.ac6_build}
: ${current_task:=Edit configuration}

# These functions could nicely fit in some utilities library
package_task_state_file() {
	local task="$1"
	local package="$2"
	[ -n "$task" ] || task="$current_task"
	[ -n "$package" ] || package="$package_name"
	task=$(echo "${task,,}" | sed -e 's/ /-/')
	local state
	if [ "$package" = "$package_name" ]; then
		state="${package_statedir}/task.${task}.${package}-${package_version}-${package_revision}"
	else
		local statelist
		statelist="../$package/$package_statedir/task.$task.$package*"
		for state in $statelist; do
			case $state in
				*.poison*)	continue										;;
				*)			if [ -r "$state" ]; then break; else state=; fi	;;
			esac
		done
	fi
	echo $state
}

poison_package_task() {
	task_state_file=$(package_task_state_file "$@")
	# Only poison a task that was already executed...
	if [ -s "$task_state_file" ]; then
		date -u +"%F %T %N" > $task_state_file.poison
	fi
}

first_executable() {
	local exe file
	for exe; do
		file="$(which $exe)"
		if [ -n "$file" -a -x "$file" ]; then
			echo "$exe"
			break
		fi
	done
}

execute_in_terminal() {
	local term=$(first_executable $possible_terminals)
	local cmd_opt="-e"
	local other_opt=""
	local quote_cmd=false
	local wait=false
	local wait_fifo_name wait_fifo
	case $term in
		gnome-terminal)
			# quote_cmd=true
			wait=true
			cmd_opt="-x"
			;;
		konsole)
			other_opt="--nofork"
			;;
	esac
	if $wait; then
		# Create a pipe I will read to wait for son to finish
		wait_fifo_name="/tmp/wait.$$"
		mkfifo $wait_fifo_name
		wait_fifo="2> $wait_fifo_name"
	fi
	if $quote_cmd; then
		$term $other_opt $cmd_opt "$* $wait_fifo"
	else
		$term $other_opt $cmd_opt /bin/bash -c "$* $wait_fifo"
	fi
	if $wait; then
		sleep 1 # Ensures the FIFO is opened for writing
		cat < $wait_fifo_name
		rm -f $wait_fifo_name
	fi
}

dot_config_hash() {
	sed -e 's/#.*$//' -e '/^$/d' "$1" | sort | uniq | sha1sum | sed -e 's/ .*$//'
}

# This task is never done... Auto-poison it
poison_package_task

old_config=$(dot_config_hash .config)
if [ "$edit_config_method" = "menuconfig" ]; then
	execute_in_terminal make $edit_config_method
else
	if make $edit_config_method; then
		true # The requested config method (xconfig or gconfig) has worked
	else
		# Xconfig nor gconfig worked (missing QT or GTK); fallback to menuconfig
		execute_in_terminal make menuconfig
	fi
fi
new_config=$(dot_config_hash .config)

# No need to poison if the compile task was not yet executed,
# nor if we don't really change the configuration file
if [ "$new_config" != "$old_config" ]; then
	poison_package_task Compile
fi
